package cn.ezhizao.project.business.controller;

import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.project.business.domain.*;
import cn.ezhizao.project.business.service.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * mrp10201Controller
 *
 * @author ezhizao_zx
 */
@RestController
@RequestMapping("/business/inc10101")
public class Inc10101Controller extends BaseController {
    @Resource
    private IInc10101Service inc10101Service;
    @Resource
    private IBizCompanyProductPackageService bizCompanyProductPackageService;
    @Resource
    private IBizProductService bizProductService;
    @Resource
    private IBizCompanyService bizCompanyService;
    @Resource
    private IBizAccessoriesService bizAccessoriesService;
    @Resource
    private IBizPullP2TimeService pullP2TimeService;

    @GetMapping("/getP2CompanyProductPackage")
    public AjaxResult getP2CompanyProductPackage() {
        // 获取所有的辅料信息
        // 和已有辅料信息对比
        return pullP2CompanyProductPackage() ? success() : error();
    }
    private boolean pullP2CompanyProductPackage() {
        BizPullP2Time pullDetail = new BizPullP2Time();
        // 加工中心
        pullDetail.setType("companyProductPackage");
        List<BizPullP2Time> pullDetails = pullP2TimeService.getList(pullDetail);
        BizPullP2Time lastPullDetail = null;
        if (pullDetails.size() > 0) {
            int lastIndex = pullDetails.size() - 1;
            lastPullDetail = pullDetails.get(lastIndex);
        }
        QueryWrapper<Inc10101> queryWrapper = new QueryWrapper<>();
        if (lastPullDetail != null) {
            Date newCondition = lastPullDetail.getPullTime();
            /*
             * 这里应该根据新系统中记录的最大的拉取日期
             * modate：P2中的修改日期，为了避免重复拉取，需要做条件判断。modate必须大于上一次拉取时间
             */
            queryWrapper.apply("modate IS NOT NULL AND modate > {0}", newCondition);
        }
        List<Inc10101> inc10101List = inc10101Service.getList(queryWrapper);
        List<BizCompanyProductPackage> bizCompanyProductPackages = bizCompanyProductPackageService.query().list();
        List<BizCompanyProductPackage> insertMachineCenter = new ArrayList<>();
        List<BizCompanyProductPackage> updateMachineCenter = new ArrayList<>();
        List<BizProduct> bizProducts = bizProductService.query().list();
        List<BizCompany> bizCompanies = bizCompanyService.query().list();
        List<BizAccessories> bizAccessories = bizAccessoriesService.query().list();

        inc10101List.forEach(l -> {
            // 判断之前是否有这个辅料
            if (bizCompanyProductPackages.stream().anyMatch(v -> v.getCompanyCode().equals(l.getCust().trim()) && v.getProductCode().equals(l.getCpart().trim()) && v.getPackageCode().equals(l.getPart().trim()))) {
                // 已插入材料
                BizCompanyProductPackage oldItem = bizCompanyProductPackages.stream().filter(v -> v.getCompanyCode().equals(l.getCust().trim()) && v.getProductCode().equals(l.getCpart().trim()) && v.getPackageCode().equals(l.getPart().trim())).findFirst().orElse(null);
                oldItem.setByInc10101(l);
                updateMachineCenter.add(oldItem);
            } else {
                BizCompanyProductPackage newItem = new BizCompanyProductPackage(l);
                if (newItem.getProductCode() != null) {
                    newItem.setProductId(bizProducts.stream().filter(v -> v.getProductCode().equals(newItem.getProductCode())).findFirst().orElse(new BizProduct()).getId());
                }
                if (newItem.getCompanyCode() != null) {
                    newItem.setCompanyId(bizCompanies.stream().filter(v -> v.getCompanyCode().equals(newItem.getCompanyCode())).findFirst().orElse(new BizCompany()).getId());
                }
                if (newItem.getPackageCode() != null) {
                    newItem.setAccessoriesId(bizAccessories.stream().filter(v -> v.getAccessoriesCode().equals(newItem.getPackageCode())).findFirst().orElse(new BizAccessories()).getId());
                }
                insertMachineCenter.add(newItem);
            }
        });

        if ((insertMachineCenter.isEmpty() || bizCompanyProductPackageService.saveBatch(insertMachineCenter)) && (updateMachineCenter.isEmpty() || bizCompanyProductPackageService.updateBatchById(updateMachineCenter))) {
            BizPullP2Time bizPullP2Time = new BizPullP2Time();
            // 将当前时间赋值给 pullTime 属性
            bizPullP2Time.setPullTime(new Date());
            bizPullP2Time.setType("companyProductPackage");
            // 保存数据
            pullP2TimeService.saveOrUpdate(bizPullP2Time);
            return true;
        }
        return false;
    }
}
