package cn.ezhizao.project.business.controller;

import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.project.business.domain.*;
import cn.ezhizao.project.business.service.IBizAccessoriesService;
import cn.ezhizao.project.business.service.IBizPullP2TimeService;
import cn.ezhizao.project.business.service.IInw10100Service;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * mrp10201Controller
 *
 * @author ezhizao_zx
 */
@RestController
@RequestMapping("/business/inw10100")
public class Inw10100Controller  extends BaseController {
    @Resource
    IInw10100Service inw10100Service;
    @Resource
    IBizAccessoriesService bizAccessoriesService;
    @Resource
    private IBizPullP2TimeService pullP2TimeService;

    @GetMapping("/getP2Accessories")
    public AjaxResult getP2Accessories() {
        // 获取所有的辅料信息
        // 和已有辅料信息对比
        return pullP2Accessories() ? success() : error();
    }

    private boolean pullP2Accessories() {
        BizPullP2Time pullDetail = new BizPullP2Time();
        // 加工中心
        pullDetail.setType("accessories");
        List<BizPullP2Time> pullDetails = pullP2TimeService.getList(pullDetail);
        BizPullP2Time lastPullDetail = null;
        if (pullDetails.size() > 0) {
            int lastIndex = pullDetails.size() - 1;
            lastPullDetail = pullDetails.get(lastIndex);
        }
        QueryWrapper<Inw10100> queryWrapper = new QueryWrapper<>();
        if (lastPullDetail != null) {
            Date newCondition = lastPullDetail.getPullTime();
            /*
             * 这里应该根据新系统中记录的最大的拉取日期
             * modate：P2中的修改日期，为了避免重复拉取，需要做条件判断。modate必须大于上一次拉取时间
             */
            queryWrapper.apply("mddate IS NOT NULL AND mddate > {0}", newCondition);
        }
        List<Inw10100> inw10100List = inw10100Service.getList(queryWrapper);
        List<BizAccessories> bizAccessories = bizAccessoriesService.query().list();
        List<BizAccessories> insertMachineCenter = new ArrayList<>();
        List<BizAccessories> updateMachineCenter = new ArrayList<>();

        inw10100List.forEach(l -> {
            // 判断之前是否有这个辅料
            if (bizAccessories.stream().anyMatch(v -> v.getAccessoriesCode().equals(l.getPart().trim()))) {
                // 已插入材料
                BizAccessories oldItem = bizAccessories.stream().filter(v -> v.getAccessoriesCode().equals(l.getPart().trim())).findFirst().orElse(null);
                oldItem.setByInw10100(l);
                updateMachineCenter.add(oldItem);
            } else {
                BizAccessories newItem = new BizAccessories(l);
                insertMachineCenter.add(newItem);
            }
        });

        if ((insertMachineCenter.isEmpty() || bizAccessoriesService.saveBatch(insertMachineCenter)) && (updateMachineCenter.isEmpty() || bizAccessoriesService.updateBatchById(updateMachineCenter))) {
            BizPullP2Time bizPullP2Time = new BizPullP2Time();
            // 将当前时间赋值给 pullTime 属性
            bizPullP2Time.setPullTime(new Date());
            bizPullP2Time.setType("accessories");
            // 保存数据
            pullP2TimeService.saveOrUpdate(bizPullP2Time);
            return true;
        }
        return false;
    }
}
